@@ -1,9 +1,9 @@
 package Test::Smoke::Smoker;
 use strict;
 
-# $Id: Smoker.pm 1039 2007-04-02 21:40:38Z abeltje $
+# $Id: Smoker.pm 1172 2008-04-18 17:53:25Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.032';
+$VERSION = '0.039';
 
 use Cwd;
 use File::Spec::Functions qw( :DEFAULT abs2rel rel2abs );
@@ -26,6 +26,8 @@ my %CONFIG = (
     df_is_vms         => $^O eq 'VMS',
     df_vmsmake        => 'MMK',
     df_harnessonly    => scalar ($^O =~ /VMS/),
+    df_hasharness3    => 0,
+    df_harness3opts   => '',
 
     df_is_win32       => $^O eq 'MSWin32',
     df_w32cc          => 'MSVC60',
@@ -34,6 +36,8 @@ my %CONFIG = (
 
     df_makeopt        => "",
     df_testmake       => undef,
+
+    df_skip_tests     => undef,
 );
 
 # Define some constants that we can use for
@@ -47,6 +51,16 @@ sub HARNESS_RE1 () {
 }
 sub HARNESS_RE2() { '^\s+(\d+(?:[-\s]+\d+)*-?)$' }
 
+sub HARNESS3_RE() {
+     '^(?:
+          (?:\ \ Failed\ tests?(?:\ number\(s\))?:\ \ )
+          |
+          \s+
+       )
+       (\d[0-9, -]*)'
+}
+
+
 =head1 NAME
 
 Test::Smoke::Smoker - OO interface to do one smoke cycle.
@@ -446,6 +460,9 @@ sub make_test_prep {
 
 sub make_test {
     my $self = shift;
+
+    $self->set_skip_tests;
+
     my( $config ) = @_;
     my $config_args = "$config";
 
@@ -456,9 +473,10 @@ sub make_test {
     my @layers = ( ($config_args =~ /-Uuseperlio\b/) || $self->{defaultenv} )
                ? qw( stdio ) : qw( stdio perlio );
 
+    my @locales = split ' ', $self->{locale};
     if ( !($config_args =~ /-Uuseperlio\b/ || $self->{defaultenv}) &&
          $self->{locale} ) {
-        push @layers, 'locale';
+        push @layers, ( 'locale' ) x @locales;
     }
 
     foreach my $perlio ( @layers ) {
@@ -477,7 +495,7 @@ sub make_test {
         } else {
             $ENV{PERL_UNICODE} = ""; # See -C in perlrun
             $ENV{LC_ALL} = $self->{locale};
-            $perlio_logmsg .= ":$self->{locale}";
+            $perlio_logmsg .= ":" . pop @locales;
         }
         $self->ttylog( "TSTENV = $perlio_logmsg\t" );
 
@@ -488,6 +506,9 @@ sub make_test {
 
 	if  ( $self->{harnessonly} ) {
 
+            $self->{harness3opts} and
+                local $ENV{HARNESS_OPTIONS} = $self->{harness3opts};
+
             $self->make_test_harness( $config );
 
         } else {
@@ -496,9 +517,10 @@ sub make_test {
 
             # MSWin32 builds from its own directory
             if ( $self->{is_win32} ) {
-                $config->has_arg( '-Uuseshrplib' )
-                    and $test_target = 'static-test';
-                $self->_run_harness_target( $test_target );
+#                $config->has_arg( '-Uuseshrplib' )
+#                    and $test_target = 'static-test';
+#                $self->_run_harness_target( $test_target );
+                $self->make_test_harness( $config );
             } else {
                 $self->_run_TEST_target( $test_target, 1 );
             }
@@ -507,6 +529,8 @@ sub make_test {
         !$had_LC_ALL && exists $ENV{LC_ALL} and delete $ENV{LC_ALL};
     }
 
+    $self->unset_skip_tests;
+
     return 1;
 }
 
@@ -539,8 +563,8 @@ sub extend_with_harness {
         my $tst_perl = catfile( curdir(), 'perl' );
         my $verbose = $self->{v} > 1 ? "-v" : "";
         my @run_harness = $self->_run( "$tst_perl harness $verbose $harness" );
-        my $harness_out = $self->_parse_harness( \%inconsistent, $all_ok,
-                                                 @run_harness );
+        my $harness_out = $self->_parse_harness_output( \%inconsistent, $all_ok,
+                                                        @run_harness );
 
         # safeguard against empty results
         $inconsistent{ $_ } ||= 'FAILED' for keys %inconsistent;
@@ -590,7 +614,11 @@ sub make_test_harness {
         $target = $config->has_arg( '-Uuseshrplib' ) ? "static-test" : "test";
     }
 
-    $self->_run_harness_target( $target, $debugging );
+    if ( $self->{hasharness3} ) {
+        $self->_run_harness3_target( $target, $debugging );
+    } else {
+        $self->_run_harness_target( $target, $debugging );
+    }
 }
 
 =item $smoker->_run_harness_target( $target, $extra )
@@ -613,20 +641,21 @@ sub _run_harness_target {
 
     my $tst = $self->_make_fork( $target, $extra );
 
-    while ( <$tst> ) {
-        $self->{v} > 1 and $self->tty( $_ );
+    my $line;
+    while ( $line = <$tst> ) {
+        $self->{v} > 1 and $self->tty( $line );
 
-        /All tests successful/ and push( @failed, $_ ), last;
+        $line =~ /All tests successful/ and push( @failed, $line ), last;
 
-        /Failed Test\s+Stat/ and $seenheader = 1, next;
+        $line =~ /Failed Test\s+Stat/ and $seenheader = 1, next;
         $seenheader or next;
 
-        my( $name, $fail ) = m/$harness_re1/;
+        my( $name, $fail ) = $line =~ m/$harness_re1/;
         if ( $name ) {
             my $dots = '.' x (40 - length $name );
             push @failed, "    $name${dots}FAILED $fail\n";
         } else {
-            ( $fail ) = m/$harness_re2/;
+            ( $fail ) = $line =~ m/$harness_re2/;
             next unless $fail;
             push @failed, " " x 51 . "$fail\n";
         }
@@ -641,6 +670,62 @@ sub _run_harness_target {
     $self->tty( "Archived results...\n" );
 }
 
+=item $smoker->_run_harness3_target( $target, $extra )
+
+The command to run C<make test_harness> differs based on platform, so
+the arguments have to be passed into general routine. C<$target>
+specifies the makefile-target, C<$makeopt> specifies the extra options
+for the make program.
+
+=cut
+
+sub _run_harness3_target {
+    my( $self, $target, $extra ) = @_;
+
+    my $harness3_re = HARNESS3_RE();
+    my $seenheader = 0;
+    my @failed = ( );
+
+    my $tst = $self->_make_fork( $target, $extra );
+
+    my $line;
+    while ( $line = <$tst> ) {
+        $self->{v} > 1 and $self->tty( $line );
+
+        $line =~ /All tests successful/ and push( @failed, $line ), last;
+
+        $line =~ /Test Summary Report/ and $seenheader = 1, next;
+        $seenheader or next;
+    
+        my( $tname ) = $line =~ /^\s*(.+\.t)\s+\(Wstat/;
+        if ( $tname ) {
+            my $dots = '.' x (60 - length $tname);
+            push @failed, "$tname${dots}FAILED\n";
+            next;
+        }
+    
+        my( $failed ) = $line =~ /$harness3_re/x;
+        if ( $failed ) {
+            push @failed, "    $failed\n";
+            next;
+        }
+    
+        my( $parse_error ) = $line =~ /^  Parse errors: (.+)/;
+        if ( $parse_error ) {
+            push @failed, "    $parse_error\n";
+            next;
+        }
+    }
+
+    close $tst or do {
+        my $error = $! || ( $? >> 8);
+        Carp::carp( "\nerror while running harness target '$target': $error" );
+    };
+
+    $self->ttylog( "\n", join( "", @failed ), "\n" );
+    $self->tty( "Archived results...\n" );
+}
+
 sub _run_TEST_target {
     my( $self, $target, $extend ) = @_;
     !$target and Carp::Confess( "No target in _run_TEST_target" );
@@ -712,8 +797,12 @@ to change.
 
 =cut
 
-sub _parse_harness {
+sub _parse_harness_output {
     my( $self, $notok, $all_ok, @lines ) = @_;
+
+    grep m/^Test Summary Report/ => @lines
+        and return $self->_parse_harness3_output( $notok, $all_ok, @lines );
+
     my $harness_re1 = HARNESS_RE1();
     my $harness_re2 = HARNESS_RE2();
 
@@ -737,6 +826,47 @@ sub _parse_harness {
     return $output;
 }
 
+=item $self->_parse_harness3_output( $\%notok, $all_ok, @lines )
+
+Fator out the parsing of the Test::Harness 3 output, as it seems subject
+to change.
+
+=cut
+
+sub _parse_harness3_output {
+    my( $self, $notok, $all_ok, @lines ) = @_;
+
+    my $harness3_re = HARNESS3_RE();
+    my $seenheader = 0;
+    my $output = join "", grep defined $_ => map {
+        my $line = $_;
+
+        my( $tname ) = $line =~ /^\s*(.+\.t)\s+\(Wstat/;
+        my( $failed ) = $line =~ /$harness3_re/x;
+        my( $parse_error ) = $line =~ /^  Parse errors: (.+)/;
+
+        if ( $tname ) {
+            delete $notok->{ $tname };
+            my $dots = '.' x (60 - length $tname);
+            "    $tname${dots}FAILED\n";
+        } elsif ( $failed ) {
+            "        $failed\n";
+        } elsif ( $parse_error ) {
+            "        $parse_error\n";
+        } else {
+            undef;
+        }
+    } grep defined $_ && length $_ => map {
+        $seenheader or $seenheader = $_ =~ /Test Summary Report/;
+        /All tests successful/ && $all_ok++;
+        $self->{v} and $self->tty( $_ );
+        $seenheader ? $_ : '';
+    } @lines;
+
+    $_[2] = $all_ok;
+    return $output;
+}
+
 =item $self->_trasnaform_testnames( @notok )
 
 C<_transform_testnames()> takes a list of testnames, as found by
@@ -768,6 +898,108 @@ sub _transform_testnames {
     return %inconsistent;
 }
 
+=item set_skip_tests( [$unset] )
+
+Read from a MANIFEST like file, set in C<< $self->{skip_tests} >>, and
+rename the files in it with the extension F<.tskip>. If C<$unset> is
+set, they will be renamed back.
+
+=item unset_skip_tests
+
+Calls C<< $self->set_skip_tests( 1 ) >>.
+
+=cut
+
+sub set_skip_tests {
+    my( $self, $unset ) = @_;
+
+    $self->{skip_tests} or return;
+    local *SKIPTESTS;
+
+    if ( open SKIPTESTS, "< $self->{skip_tests}" ) {
+        my $action = $unset ? 'Unskip' : 'Skip';
+        $self->{v} and
+            $self->tty( "$action tests from '$self->{skip_tests}'\n" );
+        my @libext;
+        my $raw;
+        while ( $raw = <SKIPTESTS> ) {
+            $raw =~ m/^\s*#/ and next;
+            $raw =~ s/(\S+).*/$1/s;
+            $raw =~ m/\.t$/ or next;
+            if ( $raw =~ m{^(?:lib|ext)/} ) {
+                push @libext, $raw;
+                next;
+            }
+            my $tsrc = File::Spec->catfile( $self->{ddir}, $raw );
+            my $tdst = $tsrc . "skip";
+            $unset and ( $tsrc, $tdst ) = ( $tdst, $tsrc );
+            -f $tsrc or next;
+            my $perms = (stat $tsrc)[2] & 07777;
+            chmod 0755, $tsrc;
+            my $did_mv = rename $tsrc, $tdst;
+            my $error = $did_mv ? "" : " ($!)";
+            $self->{v} and
+                $self->tty( sprintf "\t$raw: %sok\n", $did_mv ?  'not ' : '',
+                                    $error );
+            -f $tdst and chmod $perms, $tdst;
+        }
+        close SKIPTESTS;
+        @libext and $self->change_manifest( \@libext, $unset );
+    } else {
+        require Carp;
+        Carp::carp( "Cannot open($self->{skip_tests}): $!" );
+    }
+}
+
+sub unset_skip_tests { $_[0]->set_skip_tests( 1 ) }
+
+=item $self->change_manifest( \@tests, $unset )
+
+=cut
+
+sub change_manifest {
+    my( $self, $tests, $unset ) = @_;
+
+    my $mani_org = catfile $self->{ddir}, 'MANIFEST';
+    my $mani_new = catfile $self->{ddir}, 'MANIFEST.ORG';
+    if ( $unset ) {
+        if ( -f $mani_new ) {
+            my $perms = (stat $mani_new)[2] & 07777;
+            chmod 0755, $mani_new;
+            unlink $mani_org;
+            rename $mani_new, $mani_org;
+            chmod $perms, $mani_org;
+	}
+    } else {
+        my $perms = (stat $mani_org)[2] & 07777;
+        chmod 0755, $mani_org;
+        rename $mani_org, $mani_new or do {
+            chmod $perms, $mani_org;
+            require Carp;
+	    Carp::carp("No skip of lib or ext tests [rename($mani_new): $!]");
+            return;
+        };
+        local( *MANIO, *MANIN );
+        if ( open MANIO, "< $mani_new" ) {
+            if ( open MANIN, "> $mani_org" ) {
+                my $mline;
+                while ( $mline = <MANIO> ) {
+                    chomp $mline;
+                    ( my $fn = $mline ) =~ s/^(\S+).*/$1/;
+                    if ( ! grep /\Q$fn\E/ => @$tests ) {
+                        print MANIN "$mline\n";
+                    } else {
+                        $self->{v} and $self->tty( "\t$fn\n" );
+                    }
+                }
+                close MANIN;
+            }
+            close MANIO;
+            chmod $perms, $mani_new;
+        }
+    }
+}
+
 =item $self->_run( $command[, $sub[, @args]] )
 
 C<_run()> returns C<< qx( $command ) >> unless C<$sub> is specified.
